@@ -20,16 +20,16 @@
 #    you may not use this file except in compliance with the License.
 #    You may obtain a copy of the License at
 #    http://www.apache.org/licenses/LICENSE-2.0
-#-----------------------------------------------------------------------------
+#=============================================================================
 use     5.010_000;
 use     strict;
-#-----------------------------------------------------------------------------
-#       Tables
-#-----------------------------------------------------------------------------
+#=============================================================================
+#       Variable fields
+#=============================================================================
 package ODF::lpOD::Field;
 use base 'ODF::lpOD::Element';
-our $VERSION    = 0.1;
-use constant PACKAGE_DATE => '2010-06-23T08:52:04';
+our $VERSION    = '0.101';
+use constant PACKAGE_DATE => '2010-11-19T08:58:09';
 use ODF::lpOD::Common;
 #-----------------------------------------------------------------------------
 
@@ -56,7 +56,7 @@ sub     create
                 return FALSE;
                 }
         bless $field, __PACKAGE__;
-
+        $field->set_name($opt{name});
         if (defined $opt{text})
                 {
                 $field->set_text($opt{text});
@@ -84,8 +84,7 @@ sub     create
 sub     get_type
         {
         my $self        = shift;
-        return $self->att('office:value-type')
-                        // 'string';
+        return $self->att('office:value-type') // 'string';
         }
 
 sub     set_type
@@ -146,12 +145,13 @@ sub     set_value
         {
         my $self        = shift;
         my $value       = shift         or return undef;
-        my $type        = $self->get_type;
+        my $type        = $self->get_type();
+
         given ($type)
                 {
                 when ('string')
                         {
-                        return $self->set_text($value);
+                        $self->set_attribute('office:string-value' => shift);
                         }
                 when ('date')
                         {
@@ -159,55 +159,159 @@ sub     set_value
                                 {
                                 $value = iso_date($value);
                                 }
-                        return $self->set_att('office:date-value', $value);
+                        $self->set_att('office:date-value', $value);
                         }
                 when ('time')
                         {
-                        return $self->set_att('office:time-value', $value);
+                        $self->set_att('office:time-value', $value);
                         }
                 when (['float', 'currency', 'percentage'])
                         {
-                        return $self->set_att('office:value', $value);
+                        $self->set_att('office:value', $value);
                         }
                 when ('boolean')
                         {
-                        return $self->set_att
-                                (
+                        $self->set_att(
                                 'office:boolean-value',
                                 odf_boolean($value)
                                 );
                         }
-                }        
-        }
-
-#-----------------------------------------------------------------------------
-
-sub     get_text
-        {
-        my $self        = shift;
-        return $self->SUPER::get_text(recursive => TRUE);
+                }
+        return $self->get_value;
         }
 
+#=============================================================================
+package ODF::lpOD::Variable;
+use base 'ODF::lpOD::Field';
+our $VERSION    = '0.101';
+use constant PACKAGE_DATE => '2010-11-17T08:56:47';
+use ODF::lpOD::Common;
 #-----------------------------------------------------------------------------
 
 sub     set_text
         {
         my $self        = shift;
-        my $text        = shift;
-        my %opt         =
-                (
-                style           => undef,
-                @_
-                );
-        $self->cut_children;
-        $self->append_element
-                (odf_create_paragraph(text => $text, style => $opt{style}));
+        my $type = $self->get_type;
+        unless ($type eq 'text')
+                {
+                alert "Text not allowed for $type variables";
+                return FALSE;
+                }
+        return $self->set_attribute('office:string-value' => shift);
+        }
+
+sub     get_text
+        {
+        my $self        = shift;
+        return $self->get_attribute('office:string-value');
+        }
+
+#=============================================================================
+package ODF::lpOD::UserVariable;
+use base 'ODF::lpOD::Variable';
+our $VERSION    = '0.101';
+use constant PACKAGE_DATE => '2010-11-19T08:37:07';
+use ODF::lpOD::Common;
+#-----------------------------------------------------------------------------
+
+sub     context_path            { CONTENT, '//text:user-field-decls' }
+
+sub     create
+        {
+        return bless
+                odf_create_field('text:user-field-decl', @_), __PACKAGE__;
+        }
+
+#=============================================================================
+package ODF::lpOD::SimpleVariable;
+use base 'ODF::lpOD::Variable';
+our $VERSION    = '0.101';
+use constant PACKAGE_DATE => '2010-11-19T08:37:07';
+use ODF::lpOD::Common;
+#-----------------------------------------------------------------------------
+
+sub     context_path            { CONTENT, '//text:variable-decls' }
+
+sub     create
+        {
+        return bless
+                odf_create_field('text:variable-decl', @_), __PACKAGE__;
         }
 
 #-----------------------------------------------------------------------------
+
+sub     set_value       {}
+sub     get_value       {}
+
+#=============================================================================
+package ODF::lpOD::TextField;
+use base 'ODF::lpOD::Field';
+our $VERSION    = '0.101';
+use constant PACKAGE_DATE => '2010-11-18T17:01:05';
+use ODF::lpOD::Common;
+#-----------------------------------------------------------------------------
+
+our @TYPES =
+        (
+        'date', 'time', 'page number', 'page continuation', 'page count',
+        'sender firstname', 'sender lastname', 'sender initials',
+        'sender title', 'sender position', 'sender email',
+        'sender phone private', 'sender fax', 'sender company',
+        'sender phone work', 'sender street', 'sender postal code',
+        'sender city', 'sender country', 'sender state or province',
+        'creator', 'author name', 'author initials', 'chapter', 'file name',
+        'template name', 'sheet name', 'title', 'subject',
+        'user field get', 'variable'
+        );
+
+sub     check_type
+        {
+        my $type        = shift;
+        return ($type ~~ [@TYPES]) ? $type : FALSE;
+        }
+
+sub     types           { @TYPES }
+
+sub     classify
+        {
+        my $arg         = shift;
+        my ($tag, $elt, $class);
+        if (ref $arg)
+                {
+                $elt = $arg;
+                $tag = $elt->get_tag;
+                }
+        else
+                {
+                $tag = $arg;
+                }
+        return undef unless $tag =~ /^text:/;
+        $tag =~ s/^.*://; $tag =~ s/-/ /g;
+        if ($tag ~~ [@TYPES])
+                {
+                $class = __PACKAGE__;
+                return $elt ? bless $elt, $class : $class;
+                }
+        return undef;
+        }
+
+sub	set_class
+	{
+	my $self	= shift;
+	return classify($self);
+	}        
+
+#-----------------------------------------------------------------------------
+
+sub     get_value
+        {
+        my $self        = shift;
+        my $att = $self->get_tag() . '-value';
+        return $self->get_attribute($att) // $self->get_text;
+        }
+
+sub     set_value       { my $self = shift; $self->not_allowed }
+sub     set_text        { my $self = shift; $self->not_allowed }
+
+#=============================================================================
 1;
-
-
-
-
-
